#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# Utility classes for dealing with the COIN-OR optimization
# services
#

import os
import sys
from coopr.opt.base import results
from coopr.opt.base.formats import *
from coopr.opt import SolverResults, SolutionStatus, SolverStatus, TerminationCondition, MapContainer
from pyutilib.component.core import alias
import pyutilib.math
import OSrL

def xml_eval(vstring):
    vstring = vstring.strip()
    if vstring == 'NaN':
        return pyutilib.math.nan
    elif vstring == 'zero':
        return 0
    elif vstring == 'INF':
        return pyutilib.math.infinity
    return eval(vstring)

class ResultsReader_osrl(results.AbstractResultsReader):
    """
    Class that reads in an OSrL results file and generates a
    SolverResults object.
    """

    alias(str(ResultsFormat.osrl))

    def __init__(self, name=None):
        results.AbstractResultsReader.__init__(self,ResultsFormat.osrl)
        if not name is None:                            #pragma:nocover
            self.name = name

    def __call__(self, filename, res=None, suffixes=None):
        if res is None:
            res = SolverResults()
        osrl = OSrL.OSrL()
        osrl.read(filename)
        #
        # Process 'general' element
        #
        doc = osrl.etree.getroot()
        e = doc.find('.//{%s}general' % osrl.namespace)
        if not e is None:
            for elt in ['message', 'serviceURI', 'serviceName', 'jobID', 'solverInvoked', 'timeStamp']:
                value = e.findtext('.//{%s}%s' % (osrl.namespace,elt))
                if not value is None:
                    setattr(res.solver, elt, value)
            status = e.find('.//{%s}generalStatus' % osrl.namespace).attrib['type']
            if status == 'normal':
                res.solver.status = SolverStatus.ok
            else: 
                try:
                    res.solver.status = SolverStatus(status)
                except:
                    res.solver.status = SolverStatus.error
            res.problem.name = e.findtext('.//{%s}instanceName' % osrl.namespace)
        #
        # Process 'job' element
        #
        e = osrl.etree.find('.//{%s}job' % osrl.namespace)
        if not e is None:
            res.add('job', MapContainer(), False, "Job Execution")
            for elt in ['status', 'submitTime', 'scheduledStartTime', 'actualStartTime', 'endTime', 'usedCPUSpeed', 'usedCPUNumber', 'usedDiskSpace', 'usedMemory']:
                value = e.findtext('.//{%s}%s' % (osrl.namespace,elt))
                if not value is None:
                    setattr(res.solver, elt, value)
            e = e.find('.//{%s}time' % osrl.namespace)
            if not e is None:
                setattr(res.job, e.attrib['type'], xml_eval(e.text))
        #
        # Process 'optimization' element
        #
        e = osrl.etree.find('.//{%s}optimization' % osrl.namespace)
        if not e is None:
            res.problem.number_of_constraints = xml_eval(e.attrib['numberOfConstraints'])
            res.problem.number_of_variables = xml_eval(e.attrib['numberOfVariables'])
            res.problem.number_of_objectives = xml_eval(e.attrib['numberOfObjectives'])
            for soln in e.findall('.//{%s}solution' % osrl.namespace):
                #
                # We create a solution, and then populate its data
                # The solution object is managed by the SolverResults instance
                #
                solution = res.solution.add()
                solution.status = SolutionStatus(soln.find('.//{%s}status' % osrl.namespace).attrib['type'])
                solution.message = soln.findtext('.//{%s}message' % osrl.namespace)
                #
                vars = soln.find('.//{%s}variables' % osrl.namespace)
                if not vars is None:
                    values = vars.find('.//{%s}values' % osrl.namespace)
                    if not values is None:
                        for var in values.findall('.//{%s}var' % osrl.namespace):
                            id = int(var.attrib['idx'])
                            solution.variable['x'+str(id)] = {"Value" : xml_eval(var.text), "Id" : id}
                    #
                    values = vars.find('.//{%s}valueString' % osrl.namespace)
                    if not values is None:
                        for var in values.findall('.//{%s}var' % osrl.namespace):
                            id = int(var.attrib['idx'])
                            solution.variable['x'+str(id)] = {"Value" : xml_eval(var.text), "Id" : id}
                    #
                    for other in vars.findall('.//{%s}other' % osrl.namespace):
                        if other.attrib['name'] == 'name':                  #pragma:nocover
                            continue
                        for var in other.findall('.//{%s}var' % osrl.namespace):
                            id = int(var.attrib['idx'])
                            if not 'x'+str(id) in solution.variable:
                                solution.variable['x'+str(id)] = {"Id" : id}
                            solution.variable['x'+str(id)][other.attrib['name']] = xml_eval(var.text)
                #
                cons = soln.find('.//{%s}constraints' % osrl.namespace)
                duals = cons.find('.//{%s}dualValues' % osrl.namespace)
                if not duals is None:
                    for dual in duals.findall('.//{%s}con' % osrl.namespace):
                        id = int(dual.attrib['idx'])+1
                        solution.constraint[id].dual = xml_eval(dual.text)
                #
                objs = soln.find('.//{%s}objectives' % osrl.namespace)
                if not objs is None:
                    values = objs.find('.//{%s}values' % osrl.namespace)
                    for obj in values.findall('.//{%s}obj' % osrl.namespace):
                        try:
                            id = -int(obj.attrib['idx'])-1
                        except:
                            id = 0
                        solution.objective['x'+str(id)] = xml_eval(obj.text)
                    #
                    for other in objs.findall('.//{%s}other' % osrl.namespace):
                        if other.attrib['name'] == 'name':              #pragma:nocover
                            continue
                        for obj in other.findall('.//{%s}obj' % osrl.namespace):
                            try:
                                id = -int(obj.attrib['idx'])-1
                            except:
                                id = 0
                            if not 'x'+str(id) in solution.objective:
                                solution.objective['x'+str(id)].value = 0
                            solution.objective['x'+str(id)][other.attrib['name']] = xml_eval(obj.text)
        #
        return res



import logging
import xml.dom

from coopr.opt.base import problem, AbstractProblemWriter, ProblemFormat
import coopr.pyomo
import pyutilib

convert_name = lambda x: x.replace('[','(').replace(']',')')

logger = logging.getLogger('coopr.os')

class ProblemWriter_osil(AbstractProblemWriter):
    """
    This is a writer instance for the OSiL xml specification.  See "OSiL: An
    Instance Language for Optimization" by Robert Fourer, Jun Ma, Kipp Martin.
    Likely available at:

    http://www.coin-or.org/OS/publications/ (index of papers, Jun 2010)
    """
    pyutilib.component.core.alias('osil')

    def __init__( self ):
        AbstractProblemWriter.__init__( self, ProblemFormat.osil )


    def __call__(self, model, filename, solver_capability):
        if filename is None:
            filename = model.name + '.xml'
        #
        output = open( filename, 'w' )
        self._print_model( model, output, solver_capability )
        output.close()
        #
        return filename, None


    def _print_model( self, model, output, solver_capability ):
        dom = xml.dom.getDOMImplementation()
        doc = dom.createDocument(None, None, None)
        osil = doc.createElement('osil')
        osil.appendChild( self._getInstanceHeader( doc, model ) )
        osil.appendChild( self._getInstanceData( doc, model ) )
        doc.appendChild( osil )
        #
        output.write( doc.toprettyxml(indent="  ") )


    def _getInstanceHeader ( self, doc, model ):
        header = doc.createElement('instanceHeader')
        #
        name   = doc.createElement('name')
        name.appendChild( doc.createTextNode( model.name ) )
        header.appendChild( name )
        #
        #source = doc.createElement('source')
        #source.appendChild( doc.createTextNode( model.name ) )
        #header.appendChild( source )
        #
        description = doc.createElement('description')
        description.appendChild( doc.createTextNode( "Created OSiL problem from Pyomo model '%s'" % model.name ) )
        header.appendChild( description )
        #
        return header


    def _getInstanceData ( self, doc, model ):
        nodeFunctions = (
            self._getVariablesElement,
            self._getObjectivesElement,
            self._getConstraintsElement,
            self._getLinearConstraintCoefficientsElement,
            self._getQuadraticCoefficientsElement,
            #self._getNonLinearExpressionsElement,
            #self._getSpecialOrderedSetsElement,
        )
        #
        instanceData = doc.createElement('instanceData')
        for f in nodeFunctions:
            instanceData.appendChild( f(doc, model) )
        return instanceData


    def _getSpecialOrderedSetsElement ( self, doc, model ):
        return None


    def _getNonLinearExpressionsElement ( self, doc, model ):
        nle = doc.createElement('nonlinearExpressions')

        def create_sub_expression ( binary_action, nodes ):
            """
            Creates a sub expression of terms that need a similar action.

            For example, create_sub_expression('times', [el1, el2, el3, el4] )
            would return:

            <times><el1><times><el2><times><el3><el4></times></times></times>
            """
            if len(nodes) > 1: # multiple vars in this term
                node = doc.createElement( binary_action )
                node.appendChild( nodes.pop() )
                node.appendChild( nodes.pop() )
                for var in nodes:
                    sub_node = doc.createElement( binary_action )
                    sub_node.appendChild( node )
                    sub_node.appendChild( var )
                    node = sub_node
                expression = node
            elif len(nodes) == 1:
                expression = nodes.pop()
            else:
                # TODO include a better error message
                raise Exception, 'Pyomo internal programming error.\n'        \
                'Please inform developers of what you did to encounter this ' \
                'message: Writer, Nonlinear Sub expression creator'

            return expression


        def _create_nl_expression ( expression, index ):
            if coopr.pyomo.is_quadratic( expression ) or not coopr.pyomo.is_nonlinear( expression ):
                return None

            order_nodes = list()
            for order in sorted( expression.keys() ):
                if order in (-1, 0, 1, 2):
                    continue
                try:
                    dummy = int(order)
                except:
                    msg  = 'Unable to write OSiL format of model.\n'          \
                    'Explanation: The Pyomo OSiL Writer has not implemented ' \
                    'a nonlinear expression type that you have in your '      \
                    'objective or constraints.  (For example, sin(x), ln(x),' \
                    ' or x^(2.3).)'
                    raise Exception, msg

                order = expression[order] # get the actual expression part
                term_nodes = list()
                for term in order:
                    coef = order[term]

                    var_nodes = list()
                    for var in term:
                        vname = convert_name( expression[-1][var].name )
                        vid   = expression[-1][var].id

                        var_node = doc.createElement('variable')
                        var_node.setAttribute('name', vname )
                        var_node.setAttribute('idx', str(vid) )

                        power = term[var]
                        if power > 1:
                            raised_to = doc.createElement('number')
                            raised_to.setAttribute('value', str(power) )
                            power_node = doc.createElement('power')
                            power_node.appendChild( var_node )
                            power_node.appendChild( raised_to )
                            var_node = power_node

                        var_nodes.append( var_node )

                    sub_expr = create_sub_expression('times', var_nodes )

                    term_node = doc.createElement('times')
                    coef_node = doc.createElement('number')
                    coef_node.setAttribute('value', str(coef) )
                    term_node.appendChild( coef_node )
                    term_node.appendChild( sub_expr )

                    term_nodes.append( term_node )

                sub_expr = create_sub_expression('plus', term_nodes )
                order_nodes.append( sub_expr )

            sub_expr = create_sub_expression('plus', order_nodes )

            nl = doc.createElement('nl')
            nl.setAttribute('idx', str(index) )
            nl.appendChild( sub_expr )

            return nl


        def create_nl_expressions ( expressions_set, **kwargs ):
            index    = kwargs.pop('index', 0 )   # expression index
            modifier = kwargs.pop('modifer', 1 ) # next expression index?
            for es in expressions_set:
                for key in expressions_set[es]:
                    expression = expressions_set[es][key].repn
                    nlexpression = _create_nl_expression( expression, index )
                    if nlexpression:
                        nle.appendChild( nlexpression )

                    index += modifier

        objectives  = model.active_components( coopr.pyomo.Objective )
        constraints = model.active_components( coopr.pyomo.Constraint )
        create_nl_expressions( objectives, index=-1, modifier=-1  )
        create_nl_expressions( constraints )

        nle.setAttribute('number', str(len( nle.childNodes )) )

        return nle


    def _getLinearConstraintCoefficientsElement ( self, doc, model ):
        lcc = doc.createElement('linearConstraintCoefficients')


        # TODO: figure out when to use colIdx
        start_node = doc.createElement('start')
        col_node   = doc.createElement('colIdx')
        value_node = doc.createElement('value')

        def get_linear_info ( expression ):
            terms = []
            if 1 in expression:
                for term_id in expression[1]:
                    coef = expression[1][term_id]
                    terms.append( (term_id, coef) )
            return terms

        start = 0 # Constraint count.  OSiL orders the constraints, hence the
                # sorted version of the active_components, above
        constraints = model.active_components( coopr.pyomo.Constraint )
        for con in constraints:
            for index in sorted(constraints[con]):
                C = constraints[con][index]
                num = 0
                el_beg = doc.createElement('el')
                for col, value in get_linear_info( C.repn ):
                    num += 1
                    el_val = doc.createElement('el')
                    el_col = doc.createElement('el')
                    el_val.appendChild( doc.createTextNode( str(value)) )
                    el_col.appendChild( doc.createTextNode( str(col)) )
                    value_node.appendChild( el_val )
                    col_node.appendChild( el_col )
                el_beg.appendChild( doc.createTextNode( str(start)) )
                start_node.appendChild( el_beg )
                start += num
        lcc.appendChild( start_node )
        lcc.appendChild( col_node )
        lcc.appendChild( value_node )

        lcc.setAttribute('numberOfValues', str(len(col_node.childNodes)) )

        return lcc


    def _getConstraintsElement ( self, doc, model ):
        constraint = doc.createElement('constraints')

        def get_bound ( expression, offset=0.0 ):
            if expression.is_constant():
                return expression() + offset
            else:
                msg  = 'Non-constant constraint bound.  Expression type: '
                msg += expression.__class__.__name__
                # TODO: Convert exp.pprint to /return/ a string, not directly
                # print.  Then it's usable in any context, like below
                # if exp is not None:
                #     msg += exp.pprint()
                raise ValueError, msg

        constraints = model.active_components( coopr.pyomo.Constraint )

        for con in constraints:
            for index in sorted(constraints[con]):
                node = doc.createElement('con')
                C = constraints[con][index]

                if index is None:
                    name = convert_name( C.name )
                else:
                    name = convert_name( C.name+'(%s)' % str(index) )
                if 0 in C.repn:
                    offset = C.repn[0][None]
                else:
                    offset = 0.0

                node.setAttribute('name', name)

                if C._equality: # is an equality constraint
                    node.setAttribute('constant', str( get_bound(C.lower, -offset) ))
                else: # is an inequality constraint
                    if C.lower is not None:
                        node.setAttribute('lb', str( get_bound(C.lower, -offset) ))
                    if C.upper is not None:
                        node.setAttribute('ub', str( get_bound(C.upper, -offset) ))

                constraint.appendChild( node )

        constraint.setAttribute('numberOfConstraints', str(len(constraint.childNodes)) )

        return constraint


    def _getQuadraticCoefficientsElement ( self, doc, model ):
        quadratic = doc.createElement('quadraticCoefficients')

        qterm_index_attributes = ('idxOne', 'idxTwo')

        def add_qterms ( objs, qidx=True ):
            for idx in xrange(len(objs)):
                qterm_idx = str(idx)
                if qidx != True:
                    qterm_idx = str(-1) # -1 for objectives

                obj = objs[idx]

                for key in obj:
                    expression = obj[key].repn
                    if coopr.pyomo.is_constant( expression ):
                        # we've already informed the user we're ignoring this
                        # Object, so ignore it and move on
                        continue

                    if 2 in expression: # quadratic terms
                        keys = expression[2].keys()
                        for quadterm in tuple(keys):
                            qterm = doc.createElement('qterm')
                            i = iter( qterm_index_attributes )

                            for var_id in quadterm:
                                var_index = str( var_id[1] )
                                count = quadterm[var_id]

                                if ( 2 == count ):  # same var (e.g. X**2)
                                    qterm.setAttribute( i.next(), var_index )
                                    qterm.setAttribute( i.next(), var_index )
                                else: # different vars (e.g. X*Y)
                                    qterm.setAttribute( i.next(), var_index )

                            coef = str( expression[2][quadterm] )
                            qterm.setAttribute('coef', coef )
                            qterm.setAttribute('idx', qterm_idx )
                            quadratic.appendChild( qterm )

        # The quadratics section of the XML document deals with both
        # objectives and constraints, so we add them in two parts.
        objectives = model.active_components( coopr.pyomo.Objective )
        constraints = model.active_components( coopr.pyomo.Constraint )
        #add_qterms( objectives, False )
        #add_qterms( constraints )

        quadratic.setAttribute('numberOfQuadraticTerms', str(len(quadratic.childNodes)) )

        return quadratic


    def _getObjectivesElement ( self, doc, model ):
        #
        # don't worry about non-linear parts here.  It's checked as
        # part of _getNonlinearExpressions
        #
        objectives = doc.createElement('objectives')

        objs = model.active_components( coopr.pyomo.Objective )


        for objarray in objs:
            if objs[objarray].is_minimizing():
                sense = 'min'
            else:
                sense = 'max'

            for key in objs[objarray]: # note: None is a valid dict key
                obj = objs[objarray][key]
                expression = obj.repn
                if coopr.pyomo.is_constant( expression ):
                    msg = "Ignoring objective '%s[%s]' which is constant"
                    logger.warning( msg % (str(objarray), str(key)) )
                    continue
                #
                obj_node = doc.createElement('obj')
                #
                print expression
                if 1 in expression: # first-order terms
                    keys = sorted( expression[1].keys() )
                    for var_key in keys:
                        var_index = expression[-1][var_key].id
                        coef = expression[1][var_key]
                        value = doc.createTextNode( str(coef) )

                        coef_node = doc.createElement('coef')
                        coef_node.setAttribute('idx', str(var_index) )
                        coef_node.appendChild( value )

                        obj_node.appendChild( coef_node )

                obj_node.setAttribute('maxOrMin', sense )
                if key is None:
                    obj_node.setAttribute('name', objs[objarray].name)
                else:
                    obj_node.setAttribute('name', objs[objarray].name+'('+str(key)+')')
                obj_node.setAttribute('numberOfObjCoef', str(len(obj_node.childNodes)) )

                objectives.appendChild( obj_node )


        objectives.setAttribute('numberOfObjectives', str(len(objectives.childNodes)) )

        return objectives


    def _getVariablesElement ( self, doc, model ):
        variables = doc.createElement('variables')

        vars = model.active_components( coopr.pyomo.Var )

        for vararray in vars:
            vtable = {}
            for key in vars[vararray]:
                var = vars[vararray][key]

                if key is None:
                    name = '%s' % str(vararray)
                else:
                    name = '%s(%s)' % (str(vararray), str(key))
                name = ''.join(name.split()) # remove whitespace
                name = name.replace('((', '(').replace('))', ')')

                ubound = 'INF'
                lbound = '-INF'
                if not var.ub is None: ubound = str(var.ub.value)
                if not var.lb is None: lbound = str(var.lb.value)

                if   ( var.is_continuous() ): vtype = 'C'   # continuous
                elif ( var.is_binary() ):     vtype = 'B'   # binary
                elif ( var.is_integer() ):    vtype = 'I'   # integer
                else:                         vtype = 'S'   # string
                # TODO: can Pyomo represent 'D' (Semi-continuous) variables?
                # TODO: can Pyomo represent 'J' (Semi-integer) variables?

                node = doc.createElement('var')
                node.setAttribute('name', name )
                node.setAttribute('lb', lbound )
                node.setAttribute('ub', ubound )
                node.setAttribute('type', vtype )

                vtable[var.id] = node
            for key in sorted(vtable.keys()):
                variables.appendChild( vtable[key] )

        variables.setAttribute('numberOfVariables', str(len(variables.childNodes)) )

        return variables




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['OSrL']

import os
import sys
#from xml.dom import minidom, Node
import xml
import xml.etree.ElementTree
from pyutilib.enum import Enum
from pyutilib.misc import get_xml_text

#
# A class for reading/writing OSrL file
#
class OSrL(object):

    GeneralStatus = Enum('error', 'warning', 'success', 'normal')
    SolutionStatus = Enum('unbounded', 'globallyOptimal', 'locallyOptimal',
                        'optimal', 'bestSoFar', 'feasible', 'infeasible',
                        'stoppedByLimit', 'unsure', 'error', 'other')

    def __init__(self):
        self.solution = []
        self.etree = None
        self.namespace = 'os.optimizationservices.org'

    def write(self,ostream=None, prefix=""):
        if ostream is None:                     #pragma:nocover
            ostream = sys.stdout
        if type(ostream) is str:
            self._tmpfile = ostream
            ostream = open(self._tmpfile,"w")
        else:
            self._tmpfile = None
        self.etree.write(ostream.fileno())

    def read(self, filename):
        self.etree = xml.etree.ElementTree.parse(filename)
        self.validate()

    def validate(self):
        if self.etree is None:                  #pragma:nocover
            return
        doc = self.etree.getroot()
        e = doc.find('.//{%s}general' % self.namespace)
        e = e.find('.//{%s}generalStatus' % self.namespace)
        if e.attrib['type'] not in OSrL.GeneralStatus:
            raise ValueError, "General status value '%s' is not in OSrL.GeneralStatus" % str(e.attrib['type'])
        #
        for soln in doc.findall('.//{%s}solution' % self.namespace):
            e = soln.find('.//{%s}status' % self.namespace)
            if e.attrib['type'] not in OSrL.SolutionStatus:
                raise ValueError, "Solution status value '%s' is not in OSrL.SolutionStatus" % str(e.attrib['type'])




import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
currdir = dirname(abspath(__file__))+os.sep
datadir = currdir+'data'+os.sep+'osrlFiles'+os.sep

from nose.tools import nottest
import pyutilib.th as unittest
import coopr.opt
import pyutilib.services
import glob

pyutilib.services.TempfileManager.tempdir = currdir

class Test(unittest.TestCase):

    def setUp(self):
        pyutilib.services.TempfileManager.tempdir = currdir

    def tearDown(self):
        pyutilib.services.TempfileManager.clear_tempfiles()


@nottest
def test_osrl(self, name):
    reader = coopr.opt.ReaderFactory("osrl")
    soln = reader(datadir+name+'.osrl')
    soln.write(filename=datadir+name+'_out.json', format='json')
    self.assertMatchesJsonBaseline(datadir+name+"_out.json", datadir+name+"_baseline.json")

for fname in glob.glob(currdir+'data/osrlFiles/*.osrl'):
    name = os.path.basename(fname)
    name = name.split('.')[0]
    Test.add_fn_test(fn=test_osrl, name=name)

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

from coopr.opt.base import ProblemFormat, ResultsFormat
from coopr.opt.solver import SystemCallSolver
from pyutilib.component.core import alias
from pyutilib.misc import Bunch
from pyutilib.services import register_executable, registered_executable
from pyutilib.services import TempfileManager

create_tempfile = TempfileManager.create_tempfile

class OSSolver(SystemCallSolver):
    """The Optimization Systems solver."""

    alias('os', doc='An interface to a OS solver service')

    def __init__ (self, **kwargs):
        #
        # Call base constructor
        #
        kwargs['type'] = 'os'
        SystemCallSolver.__init__( self, **kwargs )
        #
        # Valid problem formats, and valid results for each format
        #
        self._valid_problem_formats = [ ProblemFormat.osil ]
        self._valid_result_formats  = { ProblemFormat.osil : [ResultsFormat.osrl] }

    def executable(self):
        executable = registered_executable('OSSolverService')
        if executable is None:
            pyutilib.component.core.PluginGlobals.env().log.error("Could not locate the OSSolverService executable, which is required for solver %s" % self.name)
            self.enable = False
            return None
        return executable.get_path()

    def create_command_line(self, executable, problem_files):
        #
        # Define log file
        #
        if self.log_file is None:
            self.log_file = TempfileManager.create_tempfile(suffix="_os.log")
        fname = problem_files[0]
        self.results_file = problem_files[0]+'.osrl'
        #
        options = []
        for key in self.options:
            if key == 'solver':
                continue
            elif isinstance(self.options[key],basestring) and ' ' in self.options[key]:
                opt.append('-'+key+" \""+str(self.options[key])+"\"")
            elif key == 'subsolver':
                opt.append("-solver "+str(self.options[key]))
            else:
                opt.append('-'+key+" "+str(self.options[key]))
        #
        options = ' '.join( options )
        proc = self._timer + " " + executable + " -osil " + problem_files[0] + " -osrl " + self.results_file + ' ' + options
        return Bunch(cmd=proc, log_file=None, env=None)

    def _default_results_format(self, prob_format):
        return ResultsFormat.osrl


register_executable(name='OSSolverService')



__all__ = ['OSSolverService', 'OSOptions']

import os.path
import sys


getAll = """<?xml version="1.0" encoding="UTF-8"?>
<ospl xmlns="os.optimizationservices.org">
  <processHeader>
     <request action="getAll"/>
  </processHeader>
  <processData/>
</ospl>"""

class Options(object):

    def __init__(self, name):
        self._name = name

    def __str__(self):
        tmp = '<'+self._name+'>'
        for item in dir(self):
            if item[0] != '_':
                print self._name,item
                if type(getattr(self,item)) is Options:
                    tmp += str(getattr(self,item))
                else:
                    tmp += '<'+item+'>'+str(getattr(self,item))+'</'+item+'>'
        tmp += '</'+self._name+'>'
        return tmp


class OSOptions(Options):

    def __init__(self):
        Options.__init__(self,'osol')
        self.general = Options('general')
        self.solver = Options('solver')


class OSSolverService(object):

    def __init__(self, url, verbose=False):
        self.verbose = verbose
        try:
            from suds.client import Client
        except:
            if self.verbose:
                print "WARNING: Failed to import from 'suds'"
            self.client = None
        else:
            if self.verbose:
                print "Connecting to url %s ..." % url
                
            self.client = Client(url)
            if self.verbose:
                print "... done."
        self.url = url
        print self.client

    def kill(self, osol=''):
        """Terminate a job on the server."""
        return self.client.service.kill(osol)

    def knock(self, ospl=getAll, osol=''):
        """Request process and job status information from the server in
        ospl format."""
        return self.client.service.knock(ospl, osol)

    def retrieve(self, osol=''):
        """Retrieve results from a solver.  This method
        requires a <jobID> element in the osol string."""
        return self.client.service.retrieve(osol)

    def send(self, osil, osol=''):
        """Perform asynchronous communication with a server.  This method
        requires a <jobID> element in the osol string.  It returns True if
        the problem was successfully submitted, and False otherwise."""
        return self.client.service.send(osil, osol)

    def solve(self, osil, osol='', osrl=None):
        """Perform synchronous communication with a server.  This method
        submits a model instance and waits for the solution."""
        if os.path.exists(osil):
            osil = ''.join(open(osil,'r').readlines())
        if os.path.exists(osol):
            osol = ''.join(open(osol,'r').readlines())
        results = self.client.service.solve(osil, osol)
        if not osrl is None:
            OUTPUT = open(osrl, 'w')
            print >>OUTPUT, results
            OUTPUT.close()
        else:
            return results

    def getJobID(self, osol=''):
        """Return a job ID that can be used to uniquely identify a
        job that is launched with the send() method."""
        return self.client.service.getJobID(osol)


def execute(argv=sys.argv):
    from optparse import OptionParser

    parser = OptionParser('OSSolverService [OPTIONS]')
    parser.add_option('--verbose', '-v', action='store_true', dest='verbose', default=False,
            help='Print debugging information.')
    parser.add_option('--osil', action='store', dest='osil', default=None,
            help='The name of a file that contains an optimization instance in OSiL format.')
    #parser.add_option('--nl', action='store', dest='nl', default=None,
    #        help='The name of a file that contains an optimization instance in NL format.')
    #parser.add_option('--mps', action='store', dest='mps', default=None,
    #        help='The name of a file that contains an optimization instance in MPS format.')
    parser.add_option('--osol', action='store', dest='osol', default='',
            help='The name of a file that contains solver options.')
    parser.add_option('--osrl', action='store', dest='osrl', default=None,
            help='The name of the file that will contain the solver results after optimization is performed.')
    parser.add_option('--solver', action='store', dest='solver', default=None,
            help='The name of the solver that will be used to optimize the problem.  The default value is cbc.')
    parser.add_option('--serviceLocation', '--url', action='store', dest='url', default=None,
            help='The url of the solver service.  If not specified, it is assumed that the problem is solved locally.')
    #parser.add_option('--methodName', '--method', action='store', dest='method', default=None,
    #        help='The method on the solver service that will be invoked.  Valid values are: solve (the default), send, kill, knock, getJobID, and retrieve.')
    #parser.add_option('--browser', action='store', dest='browser', default=None,
    #        help='A path to the browser on the local machine.  If this parameter is specified, then the solver result in OSrL format is transformed using XSLT into HTML and displayed in the browser.')
    #parser.add_option('--config', action='store', dest='config', default=None,
    #        help='A text files that contains values for input parameters.')

    try:
        import suds
    except:
        print "ERROR: The 'suds' package must be installed to run the OSSolverService."
        sys.exit(-1)
    options, args = parser.parse_args(argv)

    if options.url is None:
        print "ERROR: Cannot execute solver service without a service url.\n\tUse the '--url' option to specify the solver service."
        sys.exit(-1)
    if options.osil is None:
        print "ERROR: Must specify OSiL filename with the '--osil' option."
        sys.exit(-1)

    service = OSSolverService(options.url, options.verbose)
    results = service.solve(options.osil, osol=options.osol)
    if not options.osrl is None:
        print "Writing results to file '%s'" % results
        OUTPUT=open(options.osrl, 'w')
        print >>OUTPUT, results
        OUTPUT.close()
    else:
        print ""
        print "Solver Results:"
        print results
        print ""


def test():
    url = 'http://kipp.chicagobooth.edu/os/OSSolverService.jws?wsdl'
    service = OSSolverService(url)
    #print service.solve('parincLinear.osil')

    opt = OSOptions()
    opt.general.jobID = service.getJobID()
    print '1',service.send('parincLinear.osil',osol=opt)
    print '2',service.knock(osol=opt)
    print '3',service.retrieve(osol=opt)

if __name__ == '__main__':
    test()
    #execute(sys.argv)



#
# Unit Tests for coopr.os
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
currdir = dirname(abspath(__file__))+os.sep

from nose.tools import nottest
import coopr.pyomo
import coopr.opt
import coopr.os
import coopr
import xml
import pyutilib.th as unittest
import pyutilib.services

pyutilib.services.TempfileManager.tempdir = currdir

class Test(unittest.TestCase):

    def setUp(self):
        pyutilib.services.TempfileManager.tempdir = currdir
        #
        # Create OSrL object
        #
        self.osrl = coopr.os.OSrL()
        self.reader = coopr.opt.ReaderFactory("osrl")

    def tearDown(self):
        pyutilib.services.TempfileManager.clear_tempfiles()

    def test_read_solution(self):
        soln = self.reader(currdir+"test1.OSrL.xml")
        self.assertEqual(len(soln.solution),1)
        soln.write(filename=currdir+"test_os.txt", format='json')
        self.assertMatchesJsonBaseline(currdir+"test_os.txt", currdir+"test1.txt")

    def test_test2_error(self):
        try:
            self.osrl.read(currdir+"test2.OSrL.xml")
            self.fail("test_test2_error - Failed to find error in test2.OSrL.xml")
        except ValueError:
            pass

    def test_write_osil1 ( self ):
        if not 'osil' in coopr.opt.WriterFactory().services():
            self.skipTest('No OSiL writer is available.')
        base = '%s/test_write_osil1' % currdir
        fout, fbase = (base + '.out', base + '.txt')

        model = coopr.pyomo.AbstractModel()
        model.A = coopr.pyomo.RangeSet(1, 4)
        model.x = coopr.pyomo.Var( model.A, bounds=(-1,1) )
        def obj_rule ( model ):
            return coopr.pyomo.summation( model.x )
        model.obj = coopr.pyomo.Objective( rule=obj_rule )
        instance = model.create()
        instance.write( format=coopr.opt.ProblemFormat.osil, filename=fout)
        self.assertFileEqualsBaseline( fout, fbase )


if __name__ == "__main__":
    unittest.main()
